import datetime
from typing import List, Optional

from lxml.etree import XPath
from lxml.html import HtmlElement

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.data import Image
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class SRFParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 12, 3)
        _title_selector = XPath("//span[@class='article-title__text']")
        _author_selector = XPath("//span[@itemprop='author']")
        _paragraph_selector = XPath(
            "//section[@class='article-content']//span[@class='blockquote__text'] | "
            "//section[@class='article-content']/p | "
            "//section[@class='article-content']/li | "
            "//section[@class='article-content']/ul/li"
        )
        _summary_selector = XPath("//header[@class='article-header']/p[@class='article-lead']")
        # The last line of the _subheadline_selector does not comply with the format at first glance, but as far as I
        # can see, Fundus only gets a preliminary article version, when crawling live-tickers which only contains the
        # headlines of the elements. Example:
        # https://www.srf.ch/news/international/krieg-im-nahen-osten-israel-noch-unschluessig-ueber-reaktion-auf-irans-angriff
        _subheadline_selector = XPath(
            "//section[@class='article-content']/h2 | "
            "//section[@class='article-content']"
            "//div[@id='ticker']//li//span[@itemprop='headline']"
        )

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            if not (author_nodes := self._author_selector(self.precomputed.doc)):
                return []
            else:
                if len(author_nodes) > 1:
                    author_list = list()
                    for node in author_nodes:
                        for author in node.text_content().split(";"):
                            author_list.append(author)
                    return generic_author_parsing(author_list)
                return generic_author_parsing([name for name in author_nodes[0].text_content().split(";")])

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("article:published_time"))

        @attribute
        def title(self) -> Optional[str]:
            if not (title_node := self._title_selector(self.precomputed.doc)):
                return None
            else:
                node: HtmlElement = title_node[0]
                return node.text_content()

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//span[@class='media-caption__description']"),
                author_selector=XPath("./ancestor::figure//span[@class='media-caption__source']"),
                image_selector=XPath("//picture[@class='image ']//img"),
                lower_boundary_selector=XPath("(//div[@class='sharing-bar__container'])[2]"),
            )

    class V2(BaseParser):
        VALID_UNTIL = datetime.date.today()

        _title_selector = XPath("//span[@class='article-title__text']")
        _author_selector = XPath("//span[@itemprop='author']")
        _summary_selector = XPath("//p[@class='article-lead']|//ul[@class='article-list']/li")
        _paragraph_selector = XPath("//p[@class='article-paragraph']")
        _subheadline_selector = XPath("//h2[@class='article-heading']|//h3[@class='article-subheading']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            if not (author_nodes := self._author_selector(self.precomputed.doc)):
                return []
            else:
                if len(author_nodes) > 1:
                    author_list = list()
                    for node in author_nodes:
                        for author in node.text_content().split(";"):
                            author_list.append(author)
                    return generic_author_parsing(author_list)
                return generic_author_parsing([name for name in author_nodes[0].text_content().split(";")])

        @attribute
        def title(self) -> Optional[str]:
            if not (title_node := self._title_selector(self.precomputed.doc)):
                return None
            else:
                node: HtmlElement = title_node[0]
                return node.text_content()

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("datePublished"))



import datetime
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class ZwanzigMinutenParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = XPath(
            "//div[@class='Article_elementLead__N3pGr']/p | (//div[@type='typeInfoboxSummary'])[1]//li"
        )
        _subheadline_selector = XPath("//section[@class='Article_body__60Liu']//h2[contains(@class, 'crosshead')]")
        _paragraph_selector = XPath("//div[@class='Article_elementTextblockarray__WNyan']/p")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=XPath("//article"),
                caption_selector=XPath("./ancestor::figure//figcaption/span[@class='sc-d47814d6-2 bDLFoO']/p"),
                author_selector=XPath("./ancestor::figure//figcaption/span[@class='sc-d47814d6-3 bmEwwn']"),
            )



from datetime import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import (
    ArticleBody,
    BaseParser,
    Image,
    ParserProxy,
    attribute,
    utility,
)
from fundus.parser.utility import image_extraction


class KrautreporterParser(ParserProxy):
    class V1(BaseParser):
        _bloat_pattern: str = (
            "^Redaktion:|^Dieser Artikel ist eine Übersetzung|^Übersetzung:|^Recherche:|^Schlussredaktion:"
        )

        _summary_selector = CSSSelector("p[data-test='article-teaser']")
        _subheadline_selector = CSSSelector("div.article-markdown > h2")
        _paragraph_selector = XPath(
            f"//div[contains(@class, 'article-markdown')] /p[not(re:test(string(), '{_bloat_pattern}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )

        _topic_selector = XPath("string(//div[contains(@class, 'article-headers') and contains(@class, 'topic')])")

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            article_body = utility.extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )
            return article_body

        @attribute
        def authors(self) -> List[str]:
            author_string = self.precomputed.meta.get("author")
            return utility.generic_author_parsing(author_string)

        @attribute
        def publishing_date(self) -> Optional[datetime]:
            date_string = self.precomputed.ld.xpath_search("NewsArticle/datePublished", scalar=True)
            return utility.generic_date_parsing(date_string)

        @attribute
        def topics(self) -> List[str]:
            return utility.generic_topic_parsing(self._topic_selector(self.precomputed.doc))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath(
                    "//section[@class='article-headers-shared-teaser-image']//img|"
                    "//figure[contains(@class, 'image--default')]//img"
                ),
                author_selector=XPath(
                    "./ancestor::section[@class='article-headers-shared-teaser-image']"
                    "//p[@class='article-headers-shared-teaser-image__credits']"
                ),
                relative_urls=True,
            )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import (
    ArticleBody,
    BaseParser,
    Image,
    ParserProxy,
    attribute,
    function,
)
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
    transform_breaks_to_paragraphs,
)


class NikkanGeadaiParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//div[@class='article-wrap'] //p[@class='full-text'] /p[@class='br-wrap' and text()]"
        )

        _full_text_selector = CSSSelector("div.article-wrap p.full-text")

        _topic_selector = XPath("//main //div[contains(@class, 'm-keyword-list')] /ul /li //text()")

        @function(priority=0)
        def _transform_br_element(self):
            if nodes := self._full_text_selector(self.precomputed.doc):
                if len(nodes) != 1:
                    raise ValueError(f"Expected exactly one node")
                else:
                    transform_breaks_to_paragraphs(nodes[0], __class__="br-wrap")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def topics(self) -> List[str]:
            if topics := self._topic_selector(self.precomputed.doc):
                return generic_topic_parsing(topics)
            return []

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=CSSSelector("div.article-wrap"),
                # https://regex101.com/r/uY6o2z/1
                author_selector=re.compile(r"（Ｃ）(?P<credits>.*?)\s*$"),
            )



import datetime
import re
from typing import List, Optional, Pattern

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.utility import (
    apply_substitution_pattern_over_list,
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_text_extraction_with_css,
    generic_topic_parsing,
    strip_nodes_to_text,
)


class DWParser(ParserProxy):
    class V2(BaseParser):
        VALID_UNTIL = datetime.date(2024, 1, 18)
        # https://regex101.com/r/Xsadk5/1
        _author_regex = r"^([A-z]{2,3}\/)*([A-z]{2,3})\s\([A-z\s,\d]*\)$"
        _paragraph_selector = XPath(
            f"//div[contains(@class, 'rich-text')] /p[text() and not(re:test(text(), '{_author_regex}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _summary_selector = CSSSelector("header > p")
        _subheadline_selector = CSSSelector("div.rich-text > h2")

        _topic_selector = CSSSelector("aside[data-tracking-name=related-topics] > a")

        _author_substitution_pattern: Pattern[str] = re.compile(r"Deutsche Welle")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return apply_substitution_pattern_over_list(
                generic_author_parsing(self.precomputed.ld.bf_search("author")), self._author_substitution_pattern
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return [node.text_content().strip() for node in self._topic_selector(self.precomputed.doc)]

        # As of now, images cannot reliably be implemented for DW, since all pictures in the article, are loaded
        # dynamically with URLs like 'https://static.dw.com/image/65166768_${formatId}.jpg'

    class V2_1(V2):
        VALID_UNTIL = datetime.date.today()

        _topic_selector = CSSSelector("header > div.kicker > span")

        @attribute
        def topics(self) -> List[str]:
            topic_nodes = self._topic_selector(self.precomputed.doc)
            if (topic_string := strip_nodes_to_text(topic_nodes, join_on=", ")) is not None:
                return topic_string.split(", ")
            return []

    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2023, 6, 12)

        _paragraph_selector = CSSSelector("div.longText > p")
        _summary_selector = CSSSelector("p.intro")
        _subheadline_selector = CSSSelector("div.longText > h2")
        _title_selector = CSSSelector(".col3 h1")
        _author_selector = XPath(
            "normalize-space(" '//ul[@class="smallList"]' '/li[strong[contains(text(), "Auto")]]' "/text()[last()]" ")"
        )
        _date_selector = XPath(
            "normalize-space(" '//ul[@class="smallList"]' '/li[strong[contains(text(), "Datum")]]' "/text())"
        )

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            raw_author_string: str = self._author_selector(self.precomputed.doc)
            return generic_author_parsing(raw_author_string)

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            raw_date_str: str = self._date_selector(self.precomputed.doc)
            return generic_date_parsing(raw_date_str)

        @attribute
        def title(self) -> Optional[str]:
            return generic_text_extraction_with_css(self.precomputed.doc, self._title_selector)

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))



import pdb
import re
from datetime import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import (
    ArticleBody,
    BaseParser,
    Image,
    ParserProxy,
    attribute,
    function,
)
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_nodes_to_text,
    generic_topic_parsing,
    image_extraction,
    transform_breaks_to_paragraphs,
)


class MBNParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_bloat_regex = r"^\[.*\]$"

        _paragraph_selector = XPath(
            f"//div[@itemprop='articleBody']//p[(normalize-space() or @class='br-wrap') and not(re:test(string(), '{_paragraph_bloat_regex}') or @class='summary_line') and text()]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _subheadline_selector = XPath(
            "//div[@itemprop='articleBody']//p[@class='br-wrap' and not(text())]//*[self::b or (self::span and contains(@style, 'bold'))]"
        )
        _summary_selector = XPath("//div[contains(@class,'midtitle_text')]| //p[@class='summary_line']")

        _full_text_selector = XPath("//div[@itemprop='articleBody']")

        _article_author_selector = XPath("//li[@class='author']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            doc = None
            nodes = self._full_text_selector(self.precomputed.doc)
            if nodes and len(nodes) == 1:
                element = nodes[0]
                if element.xpath(".//p[normalize-space()]"):
                    for ad in element.xpath(".//div[contains(@class,'ad_wrap')]"):
                        parent = ad.getparent()
                        if parent is not None:
                            parent.remove(ad)
                if element.xpath("./self::div[@class='article_body']"):
                    # In case this uses the economy section layout, we need to transform <br> tags to paragraphs
                    doc = transform_breaks_to_paragraphs(element)
                else:
                    doc = transform_breaks_to_paragraphs(element, __class__="summary_line")
            return extract_article_body_with_selector(
                doc=doc if doc is not None else self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            if not (
                author_string := generic_author_parsing(
                    self.precomputed.ld.xpath_search("NewsArticle//author", scalar=False)
                )
            ):
                authors = generic_author_parsing(
                    generic_nodes_to_text(self._article_author_selector(self.precomputed.doc))
                )
                return [re.sub(r"\s*기자\s*", "", author) for author in authors]
            return generic_author_parsing(author_string)

        @attribute
        def publishing_date(self) -> Optional[datetime]:
            return generic_date_parsing(
                self.precomputed.ld.xpath_search("NewsArticle//datePublished", scalar=True)
                or self.precomputed.meta.get("article:published_time")
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.xpath_search("NewsArticle//headline", scalar=True) or self.precomputed.meta.get(
                "og:title"
            )

        @attribute
        def images(self) -> List[Image]:
            author_pattern = re.compile(
                r"(?P<credits>\[사진(?:\s*출처)?\s*=\s*([^\]]+)\]|<\s*([^>]+?)\s*기자\s*>|사진\s*=\s*([^.\]\r\n<>]+)|\.[^.]+$)"
            )
            images = image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=XPath("//div[@itemprop='articleBody']"),
                lower_boundary_selector=XPath("//div[@id='refTotal']"),
                image_selector=XPath(
                    "//div[@itemprop='articleBody']//div[@class='thumb_area img' or @class='image']//img"
                ),
                caption_selector=XPath(
                    "./ancestor::div[@class='thumb_area img' or @class='image']//*[(self::span and @class='thum_figure_txt') or (self::p and @class='caption')]"
                ),
                alt_selector=XPath("./@alt"),
                author_selector=author_pattern,
            )

            author_bloat_pattern = re.compile(r"\s*([.\[\]<>()]|사진(\s*출처)?\s*=|기자\s*=|사진\s*I?)\s*")
            for img in images:
                for author in img.authors:
                    authors = list()
                    author = author_bloat_pattern.sub("", author).strip()
                    if author:
                        authors.append(author)
                    img.authors = authors

            return images



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.url import NewsMap, Sitemap

from .landesspiegel import LandesspiegelParser


class LI(metaclass=PublisherGroup):
    default_language = "de"

    Landesspiegel = Publisher(
        name="Landesspiegel",
        domain="https://www.landesspiegel.li",
        parser=LandesspiegelParser,
        sources=[
            NewsMap("https://landesspiegel.li/sitemap-news.xml"),
            Sitemap("https://landesspiegel.li/sitemap-posttype-post.xml"),
        ],
    )



import datetime
import re
from typing import List, Optional, Pattern

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    apply_substitution_pattern_over_list,
    extract_article_body_with_selector,
    generic_date_parsing,
    generic_text_extraction_with_css,
    generic_topic_parsing,
    image_extraction,
)


class MDRParser(ParserProxy):
    class V1(BaseParser):
        _author_substitution_pattern: Pattern[str] = re.compile(r"MDR \w*$|MDR \w*-\w*$|MDRfragt-Redaktionsteam|^von")
        # regex examples: https://regex101.com/r/2DSjAz/1
        _source_detection: str = r"^((MDR (AKTUELL ){0,1}\(([A-z]{2,3}(\/[A-z]{2,3})*|[A-z, ]{2,50}))\)|(Quell(e|en): (u.a. ){0,1}[A-z,]{3,4})|[A-z]{2,4}(, [A-z]{2,4}){0,3}( \([A-z]{2,4}\)){0,1}$|[A-z]{2,4}\/[A-z(), \/]{3,10}$)"
        _paragraph_selector = XPath(
            f"//div[contains(@class, 'paragraph')]"
            f"/p[not(re:test(em, '{_source_detection}') or re:test(text(), '{_source_detection}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _summary_selector = CSSSelector("p.einleitung")
        _subheadline_selector = CSSSelector("div > h3.subtitle")
        _author_selector = CSSSelector(".articleMeta > .author")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def topics(self) -> List[str]:
            if self.precomputed.meta.get("news_keywords") is not None:
                return generic_topic_parsing(self.precomputed.meta.get("news_keywords"))
            else:
                return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            if raw_author_str := generic_text_extraction_with_css(self.precomputed.doc, self._author_selector):
                raw_author_str = raw_author_str.replace(" und ", ", ")
                author_list = [name.strip() for name in raw_author_str.split(",")]
                return apply_substitution_pattern_over_list(author_list, self._author_substitution_pattern)

            return []

        @attribute
        def title(self) -> Optional[str]:
            return title if isinstance(title := self.precomputed.ld.bf_search("headline"), str) else None

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=XPath("//div[@id='content']"),
                image_selector=XPath("//div[contains(@class,'mediaCon ') and not(@data-ctrl-player)]//noscript/img"),
                caption_selector=XPath("./ancestor::div[@class='media mediaA ']//span[@class='mediaSubtitle']"),
                author_selector=XPath("./ancestor::div[@class='media mediaA ']//span[@class='mediaRights copyright']"),
            )



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.data import Image
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class JyllandsPostenParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//section/article/div[contains(@class, 'c-article-inline')]"
            "/div[contains(@class, 'c-article-inline')]"
            "/div[contains(@class, 'c-article-inline')]"
            "/div/div/p | "
            "//article/p[contains(@class, '-text') and text()]"
        )
        _summary_selector = XPath("//header/p")
        _subheadline_selector = XPath(
            "//section/article/div[contains(@class, 'c-article-inline')]"
            "/div[contains(@class, 'c-article-inline')]"
            "/div[contains(@class, 'c-article-inline')]"
            "/div/div/h3 | "
            "//article/h3"
        )

        _headline_selector = XPath("//h1/text()")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            headlines: List[str] = self._headline_selector(self.precomputed.doc)
            if headlines:
                return headlines[0].strip()
            return None

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(
                self.precomputed.ld.bf_search("author") or self.precomputed.meta.get("author"), split_on=["/"]
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("article:published_time"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                author_selector=re.compile(r"\s*(Foto|Arkivfoto):\s*(?P<credits>.*)\.?"),
            )



from datetime import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class HessenschauParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = XPath(
            "//p[(@class='copytext__text text__copytext'"
            " or contains(@class, 'copytext__paragraph'))"
            " and position()=1] /strong"
        )
        _paragraph_selector = XPath(
            "//p[(@class='copytext__text text__copytext' or contains(@class, 'copytext__paragraph'))"
            " and not(child::strong and position()=1)] | "
            "//ul[contains(@class, 'copytext__paragraph')]/li"
        )
        _subheadline_selector = CSSSelector("h2[class*=head]")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def publishing_date(self) -> Optional[datetime]:
            return generic_date_parsing(self.precomputed.meta.get("article:published_time"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("news_keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure[not(@class='ar-1-1')]//*[not(self::noscript)]/img"),
                caption_selector=XPath("./ancestor::figure//span[@class='pr-3']"),
                author_selector=XPath("./ancestor::figure//span[@class='text-gray-scorpion dark:text-text-dark']"),
            )



import re
from typing import Any, Callable, Dict, Protocol

from typing_extensions import ParamSpec

P = ParamSpec("P")


def inverse(filter_func: Callable[P, bool]) -> Callable[P, bool]:
    """Logical not operator that can be used on filters

    Args:
        filter_func: The filter function to inverse.

    Returns:
        bool: boolean value of the evaluation
    """

    def __call__(*args: P.args, **kwargs: P.kwargs) -> bool:
        return not filter_func(*args, **kwargs)

    return __call__


def lor(*filters: Callable[P, bool]) -> Callable[P, bool]:
    """Logical or operator that can be used on filters

    Args:
        *filters: The filter functions to or.

    Returns:
        bool: boolean value of the evaluation
    """

    def __call__(*args: P.args, **kwargs: P.kwargs) -> bool:
        return any(f(*args, **kwargs) for f in filters)

    return __call__


def land(*filters: Callable[P, bool]) -> Callable[P, bool]:
    """Logical and operator that can be used on filters

    Args:
        *filters: The filter functions to and.

    Returns:
        bool: boolean value of the evaluation
    """

    def __call__(*args: P.args, **kwargs: P.kwargs) -> bool:
        return all(f(*args, **kwargs) for f in filters)

    return __call__


class URLFilter(Protocol):
    """Protocol to define filter used before article download.

    Filters satisfying this protocol should work inverse to build in filter(),
    so that True gets filtered and False don't.
    """

    def __call__(self, url: str) -> bool:
        """Filters a website, represented by a given <url>, on the criterion if it represents an <article>

        Args:
            url: The url the evaluation should be based on.

        Returns:
            bool: True if an <url> should be filtered out and not
                considered for extraction, False otherwise.

        """
        ...


def regex_filter(regex: str) -> URLFilter:
    def url_filter(url: str) -> bool:
        return bool(re.search(regex, url))

    return url_filter


class SupportsBool(Protocol):
    def __bool__(self) -> bool:
        ...


class ExtractionFilter(Protocol):
    """Protocol to define filters used after article extraction.

    Filters satisfying this protocol should work inverse to build in filter(),
    so that True gets filtered and False don't.
    """

    def __call__(self, extraction: Dict[str, Any]) -> SupportsBool:
        """This should implement a selection based on <extracted>.

        Extracted will be a dictionary returned by a parser mapping the attribute
        names of the parser to the extracted values.

        Args:
            extraction: The extracted values the evaluation
                should be based on.

        Returns:
            bool: True if extraction should be filtered out, False otherwise.

        """
        ...


class FilterResultWithMissingAttributes:
    def __init__(self, *attributes: str) -> None:
        self.missing_attributes = attributes

    def __bool__(self) -> bool:
        return bool(self.missing_attributes)


def _guarded_bool(value: Any):
    if isinstance(value, bool):
        return True
    else:
        return bool(value)


class Requires:
    def __init__(self, *required_attributes: str, eval_booleans: bool = True) -> None:
        """Class to filter extractions based on attribute values

        If a required_attribute is not present in the extracted data or evaluates to bool() -> False,
        this filter won't be passed. By default, required boolean attributes are evaluated with bool().

        I.e.,

            Requires("free_access")({"free_access": False}) -> will be filtered out

        You can alter this behaviour by setting `eval_bool=False`

        I.e.,

           Requires("free_access", eval_bool=False)({"free_access": False}) -> will pass

        Args:
            *required_attributes: Attributes required to evaluate to True in order to
                pass the filter. If no attributes are given, all attributes will be evaluated
            eval_bool: If True the boolean values will also be evaluated with bool(<value>).
                If False, all boolean values evaluate to True. Defaults to True.
        """
        self.required_attributes = set(required_attributes)
        # somehow mypy does not recognize bool as callable :(
        self._eval: Callable[[Any], bool] = bool if eval_booleans else _guarded_bool  # type: ignore[assignment]

    def __call__(self, extraction: Dict[str, Any]) -> FilterResultWithMissingAttributes:
        missing_attributes = [
            attribute
            for attribute in self.required_attributes or extraction.keys()
            if not self._eval(value := extraction.get(attribute)) or isinstance(value, Exception)
        ]
        return FilterResultWithMissingAttributes(*missing_attributes)


class RequiresAll(Requires):
    def __init__(self, eval_booleans: bool = False) -> None:
        """Name wrap for Requires(eval_booleans=False)

        This is for readability only. By default, it requires all non-boolean attributes of the extraction
        to evaluate to True. Set `eval_booleans=True` to include boolean values in the evaluation as well.
        See class:Requires docstring for more information.

        Args:
            eval_booleans: See Requires docstring for more information. Defaults to False.
        """
        super().__init__(eval_booleans=eval_booleans)



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class WDRParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//article//p[starts-with(@class,'text') and not(position()=last())  and not(contains(text(), 'Quelle'))]"
        )
        _summary_selector = XPath("//article//p[starts-with(@class,'einleitung')]")
        _subheadline_selector = XPath("//article//h2[@class='subtitle small']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.meta.get("Author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("Keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath(
                    "//article//picture[not(@data-resp-img-id='LinklistenteaserImageSectionZModA')]//img[@class='img']"
                ),
                upper_boundary_selector=XPath("//div[@class='segment']"),
                lower_boundary_selector=XPath("//div[@class='shareCon']"),
                alt_selector=XPath("./@title"),
                author_selector=re.compile(r"(?i)\|\s*bildquelle:(?P<credits>.+)"),
                relative_urls=True,
                caption_selector=XPath("./ancestor::div[@class='media mediaA']//p[@class='infotext']"),
            )



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath
from lxml.html import HtmlElement, fromstring, tostring

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class WinfutureParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//div[@id='news_content']/p")
        _summary_selector = XPath("//div[@id='news_content']//div[@class='teaser_text']")
        _subheadline_selector = XPath("//div[@id='news_content']/h2")

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            html_as_string = tostring(self.precomputed.doc).decode("utf-8")
            # Most paragraphs are separated by two <br>> tags, depending on if there is text
            # around those tags you need either opending or closing tags or both.
            html_as_string = re.sub(r"(<br>){2}\n<h2", "<h2", html_as_string)
            html_as_string = re.sub(r"(<br>){2}\n<(div|script)", "</p>\n<div", html_as_string)
            html_as_string = re.sub(r"(<br>){2}", "</p>\n<p>", html_as_string)
            # Subheadlines mark the beginning of a new paragraph
            html_as_string = re.sub(r"(</h2>)", "</h2>\n<p>", html_as_string)
            # Subheadlines mark the end of a paragraph
            html_as_string = re.sub(r"(?<![\W>])\n(?=<h2>)", "</p>\n", html_as_string)
            # Edge cases of paragraphs beginning/starting with various other Elements
            html_as_string = re.sub(r"(?<=<br>)\n(?!([<\W]))", "\n<p>", html_as_string)
            html_as_string = re.sub(r"(?<=(ipt|div)>)\n(?![\W<])", "\n<p>", html_as_string)
            html_as_string = re.sub(r"(?<![\W>])\n(?=<[a-z0-9=_'\"]*>)", "</p>\n", html_as_string)
            doc: HtmlElement = fromstring(html_as_string)  # type: ignore
            return extract_article_body_with_selector(
                doc=doc,
                paragraph_selector=self._paragraph_selector,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("article:published_time"))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("article:tag"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//div[@class='primary_content']//img[@class='teaser_img' or @class='photo']"),
                upper_boundary_selector=XPath("//div[@class='primary_content']"),
                lower_boundary_selector=XPath("//div[@class='mb20 more_links']"),
                caption_selector=XPath("./ancestor::span[contains(@class,'hmedia')]//a"),
                author_selector=XPath(
                    "./ancestor::div[@class='teaser_img_container']//div[@class='teaser_img_source']"
                ),
            )



import datetime
import re
from typing import List, Optional, Union

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.data import TextSequence
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class GolemParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2025, 8, 21)

        _bloat_regex = r"^Dieser Artikel enthält sogenannte Affiliate-Links"
        _summary_selector = XPath("//hgroup/p")
        _paragraph_selector = XPath(
            f"//section /p[not(@class='meta' or re:test(string(), '{_bloat_regex}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _subheadline_selector: Union[XPath, CSSSelector] = CSSSelector("div > section > h2")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                tag_filter=XPath("self::*[@class='go-vh']"),
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            if title := self.precomputed.meta.get("title"):
                return title.replace(" - Golem.de", "")
            else:
                return None

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("news_keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=XPath("//article"),
                author_selector=re.compile(r"(?i)\(bild:(?P<credits>.*)\)"),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()

        _paragraph_selector = XPath("//article//p[not(ancestor::div[@class='go-info-box__content'])]")
        _subheadline_selector = XPath("//article//h2[not(contains(@class, 'teaser'))]")
        _summary_selector = XPath("//div[@class='go-article-header__intro']")



import datetime
import pickle
from typing import Any, Dict, List, Optional, Tuple, Union

import lxml.html
import pytest

from fundus.parser.base_parser import (
    Attribute,
    AttributeCollection,
    BaseParser,
    ParserProxy,
    RegisteredFunction,
    attribute,
)
from fundus.parser.utility import generic_author_parsing
from fundus.publishers import PublisherCollection
from fundus.publishers.base_objects import Publisher
from tests.resources import attribute_annotations_mapping
from tests.utility import (
    get_meta_info_file,
    load_html_test_file_mapping,
    load_supported_publishers_markdown,
    load_test_case_data,
)


def test_supported_publishers_table():
    root = lxml.html.fromstring(load_supported_publishers_markdown())
    parsed_names: List[str] = root.xpath("//table[contains(@class,'publishers')]//td[1]/code/text()")
    for publisher in PublisherCollection:
        assert publisher.__name__ in parsed_names, (
            f"Publisher {publisher.name} is not included in docs/supported_news.md. "
            f"Run 'python -m scripts.generate_tables'"
        )


class TestBaseParser:
    def test_empty_parser(self):
        class ParserWithoutTime(BaseParser):
            pass

        ParserWithoutTime()

    def test_functions_iter(self, parser_with_function_test, parser_with_static_method):
        assert len(BaseParser.functions()) == 0
        assert len(parser_with_static_method.functions()) == 0
        assert len(parser_with_function_test.functions()) == 1
        assert parser_with_function_test.functions().names == ["test"]

    def test_attributes_iter(self, parser_with_attr_title, parser_with_static_method):
        assert len(BaseParser.attributes()) == 1
        assert len(parser_with_static_method.attributes()) == 1
        assert len(parser_with_attr_title.attributes()) == 2
        assert parser_with_attr_title.attributes().names == ["free_access", "title"]

    def test_supported_unsupported(self):
        class ParserWithValidatedAndUnvalidated(BaseParser):
            @attribute
            def validated(self) -> str:
                return "supported"

            @attribute(validate=False)
            def unvalidated(self) -> str:
                return "unsupported"

        parser = ParserWithValidatedAndUnvalidated()
        assert len(parser.attributes()) == 3

        assert (validated := parser.attributes().validated)
        assert isinstance(validated, AttributeCollection)
        assert (funcs := list(validated)) != [parser.validated]
        assert funcs[1].__func__ == parser.validated.__func__

        assert (unvalidated := parser.attributes().unvalidated)
        assert isinstance(validated, AttributeCollection)
        assert (funcs := list(unvalidated)) != [parser.unvalidated]
        assert funcs[0].__func__ == parser.unvalidated.__func__

    def test_default_values_for_attributes(self):
        class Parser(BaseParser):
            @attribute
            def test_optional(self) -> Optional[str]:
                raise Exception

            @attribute
            def test_collection(self) -> Tuple[str, ...]:
                raise Exception

            @attribute
            def test_nested_collection(self) -> List[Tuple[str, str]]:
                raise Exception

            @attribute(default_factory=lambda: "This is a default")
            def test_default_factory(self) -> Union[str, bool]:
                raise Exception

            @attribute
            def test_boolean(self) -> bool:
                raise Exception

        parser = Parser()

        default_values = {attr.__name__: attr.__default__ for attr in parser.attributes()}

        expected_values: Dict[str, Any] = {
            "test_optional": None,
            "test_collection": tuple(),
            "test_nested_collection": list(),
            "test_default_factory": "This is a default",
            "test_boolean": False,
            "free_access": False,
        }

        for name, value in default_values.items():
            assert value == expected_values[name]

        class ParserWithUnion(BaseParser):
            @attribute
            def this_should_fail(self) -> Union[str, bool]:
                raise Exception

        parser_with_union = ParserWithUnion()

        with pytest.raises(NotImplementedError):
            default_values = {attr.__name__: attr.__default__ for attr in parser_with_union.attributes()}


class TestParserProxy:
    def test_empty_proxy(self, empty_parser_proxy):
        parser_proxy = empty_parser_proxy

        with pytest.raises(ValueError):
            parser_proxy()

    def test_proxy_with_same_date(self):
        class ProxyWithSameDate(ParserProxy):
            class V2(BaseParser):
                VALID_UNTIL = datetime.date(2023, 1, 1)

            class V1(BaseParser):
                VALID_UNTIL = datetime.date(2023, 1, 1)

        with pytest.raises(ValueError):
            ProxyWithSameDate()

    def test_len(self, proxy_with_two_versions_and_different_attrs):
        parser_proxy = proxy_with_two_versions_and_different_attrs()
        assert len(parser_proxy) == 2

    def test_iter(self, proxy_with_two_versions_and_different_attrs):
        versioned_parsers = list(proxy_with_two_versions_and_different_attrs())
        assert versioned_parsers[0].VALID_UNTIL > versioned_parsers[1].VALID_UNTIL

    def test_latest(self, proxy_with_two_versions_and_different_attrs):
        parser_proxy = proxy_with_two_versions_and_different_attrs()
        print(parser_proxy.latest_version.__name__)
        assert parser_proxy.latest_version == parser_proxy.Later

    def test_call(self, proxy_with_two_versions_and_different_attrs):
        parser_proxy = proxy_with_two_versions_and_different_attrs()
        assert type(parser_proxy()) == parser_proxy.latest_version

        for versioned_parser in parser_proxy:
            from_proxy = parser_proxy(versioned_parser.VALID_UNTIL)
            assert isinstance(from_proxy, versioned_parser)
            assert from_proxy == parser_proxy(versioned_parser.VALID_UNTIL)

    def test_mapping(self, proxy_with_two_versions_and_different_attrs):
        parser_proxy = proxy_with_two_versions_and_different_attrs()

        for versioned_parser in parser_proxy:
            assert versioned_parser.attributes() == parser_proxy.attribute_mapping[versioned_parser]

        attrs1, attrs2 = parser_proxy.attribute_mapping.values()
        assert attrs1.names != attrs2.names

    def test_deprecated(self, proxy_with_two_deprecated_attributes):
        def get_initialized_attrs(parser: BaseParser) -> List[RegisteredFunction]:
            return parser._sorted_registered_functions

        proxy: ParserProxy = proxy_with_two_deprecated_attributes()

        number_of_attributes = len(proxy.latest_version.attributes(include_all=True))

        parser1 = proxy(datetime.date(2023, 1, 1))
        assert len(parser1.registered) == number_of_attributes

        parser2 = proxy(datetime.date(2024, 3, 1))
        assert len(parser2.registered) == number_of_attributes - 1

        parser3 = proxy(datetime.date(2024, 4, 1))
        assert len(parser3.registered) == number_of_attributes - 2

        assert parser3 == proxy(datetime.date(2024, 5, 1))
        assert parser3 != parser2 != parser1


# enforce test coverage for test parsing
# because this is also used for the generate_parser_test_files script we export it here
attributes_required_to_cover = {"title", "authors", "topics", "publishing_date", "body", "images"}

attributes_parsers_are_required_to_cover = {"body"}


@pytest.mark.parametrize(
    "publisher", list(PublisherCollection), ids=[publisher.__name__ for publisher in PublisherCollection]
)
class TestParser:
    def test_annotations(self, publisher: Publisher) -> None:
        parser_proxy = publisher.parser
        for versioned_parser in parser_proxy:
            assert attributes_parsers_are_required_to_cover.issubset(
                set(versioned_parser.attributes().validated.names)
            ), f"{versioned_parser.__name__!r} should implement at least {attributes_parsers_are_required_to_cover!r}"
            for attr in versioned_parser.attributes().validated:
                if annotation := attribute_annotations_mapping[attr.__name__]:
                    assert attr.__annotations__.get("return") == annotation, (
                        f"Attribute {attr.__name__!r} for {versioned_parser.__name__!r} is of wrong type. "
                        f"{attr.__annotations__.get('return')} != {annotation}"
                    )
                else:
                    raise KeyError(f"Unsupported attribute {attr.__name__!r}")

    def test_parsing(self, publisher: Publisher) -> None:
        comparative_data = load_test_case_data(publisher)
        html_mapping = load_html_test_file_mapping(publisher)

        for versioned_parser in publisher.parser:
            # validate json
            version_name = versioned_parser.__name__
            assert (
                version_data := comparative_data.get(version_name)
            ), f"Missing test data for parser version {version_name!r}"

            for key, value in version_data.items():
                if not value:
                    raise ValueError(
                        f"There is no value set for key {key!r} in the test JSON. "
                        f"Only complete articles should be used as test cases"
                    )

            # test coverage
            supported_attrs = set(versioned_parser.attributes().names)
            missing_attrs = attributes_required_to_cover & supported_attrs - set(version_data.keys())
            assert (
                not missing_attrs
            ), f"Test JSON for {version_name} of publisher {publisher.name} does not cover the following attribute(s): {missing_attrs}"

            assert list(version_data.keys()) == sorted(
                attributes_required_to_cover & supported_attrs
            ), f"Test JSON for {version_name} is not in alphabetical order"

            assert (
                html := html_mapping.get(versioned_parser)
            ), f"Missing test HTML for parser version {version_name} of publisher {publisher.name}"
            # compare data
            extraction = versioned_parser().parse(html.content, "raise")
            for key, value in version_data.items():
                assert value == extraction[key], f"{key!r} is not equal"

            # check if extraction is pickable
            pickle.dumps(extraction)

    def test_reserved_attribute_names(self, publisher: Publisher):
        parser = publisher.parser
        for attr in attribute_annotations_mapping.keys():
            if value := getattr(parser, attr, None):
                assert isinstance(value, Attribute), f"The name {attr!r} is reserved for attributes only."


class TestUtility:
    def test_generic_author_parsing(self):
        # type None
        assert generic_author_parsing(None) == []

        # type str
        assert generic_author_parsing("Peter") == ["Peter"]
        assert generic_author_parsing("Peter , Funny") == ["Peter", "Funny"]
        test_string = "Peter und Funny and the seven dandelions, right;?,;"
        assert generic_author_parsing(test_string) == ["Peter", "Funny", "the seven dandelions", "right", "?"]
        assert generic_author_parsing(test_string, split_on=["Funny", "dandelions"]) == [
            "Peter und",
            "and the seven",
            ", right;?,;",
        ]

        # type dict
        assert generic_author_parsing({"name": "Peter Funny"}) == ["Peter Funny"]
        assert generic_author_parsing({}) == generic_author_parsing({"what": "ever"}) == []

        # type list[str]
        assert generic_author_parsing([" Now", "we", "test ", " strip ", " ping ? "]) == [
            "Now",
            "we",
            "test",
            "strip",
            "ping ?",
        ]

        # type list[dict]
        assert generic_author_parsing(
            [{"name": "Peter Funny"}, {"name": "Funny Peter"}, {"this": "is not a pipe"}, {}]  # type: ignore
        ) == ["Peter Funny", "Funny Peter"]
        assert generic_author_parsing([{}]) == generic_author_parsing([{}, {"wrong": "key"}]) == []  # type: ignore


class TestMetaInfo:
    def test_order(self):
        for cc in PublisherCollection.get_subgroup_mapping().values():
            meta_file = get_meta_info_file(cc)
            meta_info = meta_file.load()
            assert meta_info, f"Meta info file {meta_file.path} is missing"
            assert sorted(meta_info.keys()) == list(meta_info.keys()), (
                f"Meta info file {meta_file.path} " f"isn't ordered properly."
            )



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.publishers.my.malay_mail import MalayMailParser
from fundus.scraping.url import RSSFeed, Sitemap


class MY(metaclass=PublisherGroup):
    default_language = "ms"

    MalayMail = Publisher(
        name="Malay Mail",
        domain="https://www.malaymail.com/",
        parser=MalayMailParser,
        sources=[
            Sitemap("https://www.malaymail.com/sitemap.xml"),
            RSSFeed("https://www.malaymail.com/feed/rss/"),
        ],
    )



import pytest

from fundus import Crawler, NewsMap, RSSFeed


class TestPipeline:
    def test_crawler_with_empty_collection(self, group_with_empty_publisher_subgroup):
        with pytest.raises(ValueError):
            Crawler(group_with_empty_publisher_subgroup)

        with pytest.raises(ValueError):
            Crawler(*group_with_empty_publisher_subgroup)

    def test_crawler_with_collection(self, group_with_valid_publisher_subgroup):
        crawler = Crawler(*group_with_valid_publisher_subgroup)
        assert len(crawler.publishers) == 1

    def test_crawler_with_two_collections(
        self,
        group_with_valid_publisher_subgroup,
        group_with_empty_publisher_subgroup,
        group_with_two_valid_publisher_subgroups,
    ):
        crawler = Crawler(group_with_empty_publisher_subgroup, group_with_valid_publisher_subgroup)
        assert len(crawler.publishers) == 1

        crawler = Crawler(group_with_valid_publisher_subgroup, group_with_valid_publisher_subgroup)
        assert len(crawler.publishers) == 1

        crawler = Crawler(group_with_two_valid_publisher_subgroups)
        assert len(crawler.publishers) == 2

        crawler = Crawler(group_with_valid_publisher_subgroup, group_with_two_valid_publisher_subgroups)
        assert len(crawler.publishers) == 3

    def test_crawler_with_publisher_enum(self, publisher_group_with_rss_feeds, publisher_group_with_news_map):
        crawler = Crawler(publisher_group_with_rss_feeds, publisher_group_with_news_map)
        assert len(crawler.publishers) == 2

        crawler = Crawler(publisher_group_with_rss_feeds, publisher_group_with_news_map, restrict_sources_to=[RSSFeed])
        assert len(crawler.publishers) == 2

        crawler = Crawler(publisher_group_with_rss_feeds, publisher_group_with_news_map, restrict_sources_to=[NewsMap])
        assert len(crawler.publishers) == 2

    def test_consecutive_calls_to_crawl(self, group_with_valid_publisher_subgroup):
        crawler = Crawler(group_with_valid_publisher_subgroup)
        next(crawler.crawl(max_articles=0), None)
        next(crawler.crawl(max_articles=0), None)



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class TazParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 10, 14)
        _paragraph_selector = CSSSelector(".sectbody > p[class*='article']")
        _summary_selector = CSSSelector(".intro")
        _subheadline_selector = CSSSelector(".sectbody > h6")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("taz:title")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.meta.get("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("article:published_time"))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

    class V2(BaseParser):
        _paragraph_selector = CSSSelector(".main-article-corpus p.bodytext.paragraph")
        _summary_selector = CSSSelector("article > p.column.typo-r-subline-detail")
        _subheadline_selector = CSSSelector(".main-article-corpus h2")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("taz:title")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("taz:tag"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//div[@data-for='picture']//img[not(@class)]"),
                caption_selector=XPath("./ancestor::div[@data-for='picture']/figcaption/text()"),
                author_selector=XPath("./ancestor::div[@data-for='picture']/figcaption/span"),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
    parse_title_from_root,
    strip_nodes_to_text,
)


class FAZParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 4, 15)

        _paragraph_selector = CSSSelector("div.atc-Text > p")
        _summary_selector = CSSSelector("div.atc-Intro > p")
        _subheadline_selector = CSSSelector("div.atc-Text > h3")
        _author_selector = CSSSelector(".atc-MetaAuthor")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            # Unfortunately, the raw data may contain cities. Most of these methods aims to remove the cities heuristically.
            if not (author_nodes := self._author_selector(self.precomputed.doc)):
                return []
            else:
                if len(author_nodes) > 1:
                    # With more than one entry, we abuse the fact that authors are linked with an <a> tag,
                    # but cities are not
                    author_nodes = [node for node in author_nodes if next(node.iterchildren(tag="a"), None) is not None]
                return [text for node in author_nodes if "F.A.Z" not in (text := node.text_content())]

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        # As of now, images can't be implemented for FAZ, since they are not crawled by CC-Bot

    class V2(BaseParser):
        VALID_UNTIL = datetime.date(2025, 2, 26)

        _summary_selector = CSSSelector("div.header-teaser")
        _paragraph_selector = CSSSelector(".body-elements__paragraph")
        _subheadline_selector = CSSSelector("div.body-elements > h3")

        _author_meta_selector = CSSSelector("div.author-meta")
        _topic_selector = XPath("//div[text()=' Schlagworte: '] /a")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def topics(self) -> List[str]:
            topic_string = strip_nodes_to_text(self._topic_selector(self.precomputed.doc), join_on=",")
            if topic_string is not None:
                topic_string = topic_string.replace(",Alle Themen", "")
                return generic_topic_parsing(topic_string, delimiter=",")
            return []

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            if self._author_meta_selector(self.precomputed.doc):
                return generic_author_parsing(self.precomputed.ld.bf_search("author"))
            return []

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title") or parse_title_from_root(self.precomputed.doc)

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure//img|//picture//img"),
                caption_selector=XPath("./ancestor::figure//span"),
                author_selector=XPath("./ancestor::figure//em"),
            )

    class V3(BaseParser):
        _summary_selector = CSSSelector("p[data-external-selector='header-teaser']")
        _paragraph_selector = XPath("//*[@data-selector='body-paragraph']")
        _subheadline_selector = CSSSelector("div[data-external-selector='body-elements'] > div > h3")

        _topic_selector = CSSSelector("nav[aria-label='Themen in diesem Artikel'] a")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def topics(self) -> List[str]:
            topic_string = strip_nodes_to_text(self._topic_selector(self.precomputed.doc), join_on=",")
            if topic_string is not None:
                topic_string = topic_string.replace(",Alle Themen", "")
                return generic_topic_parsing(topic_string, delimiter=",")
            return []

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title") or parse_title_from_root(self.precomputed.doc)

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure//img|//picture//img"),
                caption_selector=XPath(
                    "./ancestor::figure//span | ./ancestor::div[@data-external-selector='article-header']//span[@class='meta2 pr-[10px]']"
                ),
                author_selector=XPath(
                    "./ancestor::figure//*[self::em or self::small] | ./ancestor::div[@data-external-selector='article-header']//small"
                ),
            )



import bz2
import gzip
import itertools
import lzma
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from functools import cached_property
from typing import Callable, ClassVar, Dict, Iterable, Iterator, List, Optional, Set
from urllib.parse import unquote

import feedparser
import lxml.html
import validators
from lxml.etree import XMLParser, XPath
from requests import ConnectionError, HTTPError, ReadTimeout

from fundus.logging import create_logger
from fundus.scraping.filter import URLFilter, inverse
from fundus.scraping.session import _default_header, session_handler

logger = create_logger(__name__)


class CompressionFormat:
    def __init__(
        self, name: str, decompression: Optional[Callable[[bytes], bytes]] = None, *, byte_mask: Optional[bytes] = None
    ) -> None:
        self.name = name
        self.decompression = decompression
        self.byte_mask = byte_mask

    def match(self, compressed_content: bytes) -> bool:
        if self.byte_mask:
            return compressed_content.startswith(self.byte_mask)
        return False

    def __call__(self, compressed_content: bytes) -> bytes:
        if self.decompression is None:
            raise NotImplementedError(f"Decompression not implemented for {self.name!r}")
        return self.decompression(compressed_content)

    def __repr__(self):
        if self.decompression is None:
            return f"{self.name} -- Not implemented"
        return self.name


class CompressionFormats:
    GZIP = CompressionFormat("gzip", gzip.decompress, byte_mask=b"\x1f\x8b")
    BZ2 = CompressionFormat("bz2", bz2.decompress, byte_mask=b"\x42\x5a")
    ZIP = CompressionFormat("zip", byte_mask=b"PK\x03\x04")
    LZMA = CompressionFormat("lzma", lzma.decompress, byte_mask=b"\x28\xb5\x2f\xfd")

    @classmethod
    def iter_formats(cls) -> Iterator[CompressionFormat]:
        for obj in cls.__dict__.values():
            if isinstance(obj, CompressionFormat):
                yield obj

    @classmethod
    def identify(cls, compressed_content: bytes) -> Optional[CompressionFormat]:
        for compression_format in cls.iter_formats():
            if compression_format.match(compressed_content):
                return compression_format
        return None


class _ArchiveDecompressor:
    def __init__(self):
        self.archive_mapping: Dict[str, Callable[[bytes], bytes]] = {
            "application/octet-stream": self._decompress_octet_stream,
            "application/x-gzip": CompressionFormats.GZIP,
            "gzip": CompressionFormats.GZIP,
        }

    def _decompress_octet_stream(self, compressed_content: bytes) -> bytes:
        if (compression_format := CompressionFormats.identify(compressed_content)) is None:
            logger.debug(f"Could not identify compression format")
            raise NotImplementedError

        return compression_format(compressed_content)

    def decompress(self, content: bytes, file_format: "str") -> bytes:
        decompress_function = self.archive_mapping[file_format]
        return decompress_function(content)

    @cached_property
    def supported_file_formats(self) -> List[str]:
        return list(self.archive_mapping.keys())


def clean_url(url: str) -> str:
    return unquote(url)


@dataclass
class URLSource(Iterable[str], ABC):
    url: str
    languages: Set[str] = field(default_factory=set)

    _request_header: Dict[str, str] = field(default_factory=dict)

    def __post_init__(self):
        if not self._request_header:
            self._request_header = _default_header
        if not validators.url(self.url, strict_query=False):
            logger.error(f"{type(self).__name__} initialized with invalid URL {self.url}")

    def set_header(self, request_header: Dict[str, str]) -> None:
        self._request_header = request_header

    @abstractmethod
    def __iter__(self) -> Iterator[str]:
        raise NotImplemented

    def get_urls(self, max_urls: Optional[int] = None) -> Iterator[str]:
        """Returns a generator yielding up to <max_urls> URLs from <self>.


        Args:
            max_urls (int): Number of max URLs to return. Set value is
                an upper bound and not necessarily the actual number of
                URLs. If set to None, the source will be exhausted until
                StopIteration is hit. Defaults to None.

        Yields:
            str: The next URL.
        """
        return itertools.islice(self, max_urls)


@dataclass
class RSSFeed(URLSource):
    def __iter__(self) -> Iterator[str]:
        session = session_handler.get_session()
        try:
            response = session.get_with_interrupt(self.url, headers=self._request_header)

        except (HTTPError, ConnectionError, ReadTimeout) as err:
            logger.warning(f"Warning! Couldn't parse rss feed {self.url!r} because of {err}")
            return

        except Exception as error:
            logger.error(f"Warning! Couldn't parse rss feed {self.url!r} because of an unexpected error {error!r}")
            return

        html = response.text
        rss_feed = feedparser.parse(html)
        if exception := rss_feed.get("bozo_exception"):
            logger.warning(f"Warning! Couldn't parse rss feed {self.url!r} because of {exception}")
            return
        else:
            urls = filter(bool, (entry.get("link") for entry in rss_feed["entries"]))
            for url in urls:
                yield clean_url(url)


@dataclass
class Sitemap(URLSource):
    recursive: bool = True
    reverse: bool = False
    sitemap_filter: URLFilter = lambda url: not bool(url)

    _decompressor: ClassVar[_ArchiveDecompressor] = _ArchiveDecompressor()
    _sitemap_selector: ClassVar[XPath] = XPath("//*[local-name()='sitemap']/*[local-name()='loc']")
    _url_selector: ClassVar[XPath] = XPath("//*[local-name()='url']/*[local-name()='loc']")
    _parser = XMLParser(strip_cdata=False)

    def __iter__(self) -> Iterator[str]:
        def yield_recursive(sitemap_url: str) -> Iterator[str]:
            session = session_handler.get_session()
            if not validators.url(sitemap_url):
                logger.info(f"Skipped sitemap {sitemap_url!r} because the URL is malformed")
            try:
                response = session.get_with_interrupt(url=sitemap_url, headers=self._request_header)

            except (HTTPError, ConnectionError, ReadTimeout) as error:
                logger.warning(f"Warning! Couldn't reach sitemap {sitemap_url!r} because of {error!r}")
                return

            except Exception as error:
                logger.error(
                    f"Warning! Couldn't reach sitemap {sitemap_url!r} because of an unexpected error {error!r}"
                )
                return

            content = response.content.strip()
            if (content_type := response.headers.get("content-type")) in self._decompressor.supported_file_formats:
                try:
                    content = self._decompressor.decompress(content, content_type)
                except NotImplementedError:
                    logger.warning(f"No matching decompression found for {sitemap_url!r}")
                    return
            if not content:
                logger.warning(f"Warning! Empty sitemap at {sitemap_url!r}")
                return
            tree = lxml.etree.fromstring(content, parser=self._parser)
            urls = [node.text for node in self._url_selector(tree)]
            if urls:
                for new_url in reversed(urls) if self.reverse else urls:
                    yield clean_url(new_url)
            elif self.recursive:
                sitemap_locs = [node.text for node in self._sitemap_selector(tree)]
                filtered_locs = list(filter(inverse(self.sitemap_filter), sitemap_locs))
                for loc in reversed(filtered_locs) if self.reverse else filtered_locs:
                    yield from yield_recursive(loc)

        yield from yield_recursive(self.url)


@dataclass
class NewsMap(Sitemap):
    pass



from datetime import datetime, timedelta

from dateutil.rrule import MONTHLY, rrule

from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.publishers.it.corriere_della_sera import CorriereDellaSeraParser
from fundus.publishers.it.il_giornale import IlGiornaleParser
from fundus.publishers.it.la_repubblica import LaRepubblicaParser
from fundus.publishers.it.tageszeitung import TageszeitungParser
from fundus.scraping.filter import regex_filter
from fundus.scraping.url import NewsMap, RSSFeed, Sitemap


class IT(metaclass=PublisherGroup):
    default_language = "it"

    LaRepubblica = Publisher(
        name="La Repubblica",
        domain="https://www.repubblica.it",
        parser=LaRepubblicaParser,
        sources=[
            RSSFeed("https://www.repubblica.it/rss/homepage/rss2.0.xml"),
        ]
        + [
            Sitemap(f"https://www.repubblica.it/sitemap-{date.strftime('%Y-%m')}.xml")
            for date in reversed(
                list(rrule(MONTHLY, dtstart=datetime(2020, 1, 1), until=datetime.now() + timedelta(days=30)))
            )
        ],
    )

    CorriereDellaSera = Publisher(
        name="Corriere della Sera",
        domain="https://www.corriere.it",
        parser=CorriereDellaSeraParser,
        sources=[
            # Main RSS feeds
            RSSFeed("https://www.corriere.it/rss/homepage.xml"),
            RSSFeed("https://www.corriere.it/rss/ultimora.xml"),  ## Current empty but could be in use
            RSSFeed("https://www.corriere.it/dynamic-feed/rss/section/Dataroom.xml"),
            RSSFeed("https://www.corriere.it/dynamic-feed/rss/section/lettere-al-direttore.xml"),
            RSSFeed("https://www.corriere.it/dynamic-feed/rss/section/lo-dico-al-corriere.xml"),
            RSSFeed("https://www.corriere.it/dynamic-feed/rss/section/frammenti-di-ferruccio-de-bortoli.xml"),
            # Main sitemaps
            Sitemap("https://www.corriere.it/rss/sitemap_v2.xml"),
            # Dynamic sitemaps - Last 100 articles
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Economia.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Salute.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Scienze.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Interni.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Esteri.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Sport.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Politica.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Salute__Figli__e__Genitori.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Salute__Sportello__Cancro.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Elezioni.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Tecnologia.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Offerte__recensioni.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Lotterie.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Spettacoli.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Scuola.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Animali.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Opinioni.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Caffe-gramellini.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Ultimo-banco.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Letti-da-rifarei.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Piccole-dosi.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/L-angolo.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Padiglione-italia.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Facce-nuove.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Ritorno-in-solferino.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Oriente-occidente.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Sette.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Moda.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/BuoneNotizie.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/lettere__al__direttore.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/lo__dico__al__corriere.xml"),
            Sitemap("https://www.corriere.it/dynamic-sitemap/sitemap-last-100/Frammenti-ferruccio-de-bortoli.xml"),
            # Section sitemaps
            Sitemap("https://www.corriere.it/rss/sitemap/Motori.xml"),
            Sitemap("https://www.corriere.it/rss/sitemap/Cultura.xml"),
            Sitemap("https://www.corriere.it/rss/sitemap/lettere-al-direttore.xml"),
            Sitemap("https://www.corriere.it/rss/sitemap/lo-dico-al-corriere.xml"),
            Sitemap("https://www.corriere.it/rss/sitemap/Cook-Last.xml"),
        ],
    )

    IlGiornale = Publisher(
        name="Il Giornale",
        domain="https://www.ilgiornale.it",
        parser=IlGiornaleParser,
        sources=[
            # Main RSS feed (removed the one returning 404)
            RSSFeed("https://www.ilgiornale.it/feed.xml"),
            # Main sitemaps - excluding video and image sitemaps
            NewsMap("https://www.ilgiornale.it/sitemap/google-news.xml"),
            Sitemap(
                "https://www.ilgiornale.it/sitemap/indice.xml",
                sitemap_filter=regex_filter(r"\*/video/|\*/image/"),
            ),
        ],
    )

    Tageszeitung = Publisher(
        name="Die Neue Südtiroler Tageszeitung",
        domain="https://www.tageszeitung.it",
        parser=TageszeitungParser,
        sources=[
            Sitemap(
                "https://www.tageszeitung.it/sitemap.xml",
                sitemap_filter=regex_filter(r"page|misc|\/sitemap\.xml"),
                languages={"de"},
            ),
        ],
    )



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class TimesLiveParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//div[@class='wrap']//div[@class='text']/p[span or text()]")
        _summary_selector = XPath("//h3[contains(@class, 'article-title-tertiary')] ")
        _subheadline_selector = XPath("//div[@class='wrap']//div[@class='text']/h3")

        _bloat_topics = [
            "reuters",
            "timeslive",
            "Breaking news",
            "general",
            "politics",
            "sport",
            "entertainment",
            "lifestyle",
            "weird",
            "world",
            "africa",
            "news",
            "extra",
            "Sunday times",
            "times",
            "the times",
            "business times",
            "tshisa live",
        ]

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return [
                t for t in generic_topic_parsing(self.precomputed.meta.get("keywords")) if t not in self._bloat_topics
            ]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                lower_boundary_selector=XPath("//div[@class='wrap']//hr"),
                upper_boundary_selector=XPath("//h1"),
                image_selector=XPath("//div[contains(@class, 'image-container')]//img"),
                caption_selector=XPath(
                    "./ancestor::div[contains(@class, 'image-container')]//span[@class='description']"
                ),
                author_selector=XPath("./ancestor::div[contains(@class, 'image-container')]//span[@class='name']"),
                relative_urls=True,
            )



import datetime
import re
from typing import List, Optional, Pattern

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    apply_substitution_pattern_over_list,
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class NTVParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 8, 1)
        _author_substitution_pattern: Pattern[str] = re.compile(r"n-tv NACHRICHTEN")
        _summary_selector = XPath("//div[@class='article__text']/p[not(last()) and strong][1]")
        _paragraph_selector = XPath(
            "//div[@class='article__text']" "/p[not(strong) or (strong and (position() > 1 or last()))]"
        )
        _subheadline_selector = CSSSelector(".article__text > h2")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            initial_list = generic_author_parsing(self.precomputed.meta.get("author"))
            return apply_substitution_pattern_over_list(initial_list, self._author_substitution_pattern)

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("date"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure[not(contains(@class, 'teaser'))]//picture/img"),
                upper_boundary_selector=XPath("//article[@class='article']"),
                caption_selector=XPath("./ancestor::figure//figcaption/p[@class='article__caption']"),
                author_selector=XPath("./ancestor::figure//figcaption/p[@class='article__credit']"),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()
        _author_selector = XPath("string(//span[@class='article__author'])")

        @attribute
        def authors(self) -> List[str]:
            author_text: str = self._author_selector(self.precomputed.doc)
            return generic_author_parsing(author_text.replace("Von", ""))



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
    strip_nodes_to_text,
)


class NieuwsbladParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = XPath("//*[@data-testid='article-intro']")
        _paragraph_selector = XPath("//*[@data-testid='article-body']/p[text()]")
        _subheadline_selector = XPath(
            "//*[@data-testid='article-body']/p/span[@class='bold'] | " "//*[@data-testid='article-body']/h3"
        )

        _topic_selector = XPath("//ul[contains(@class, 'taglist')]/li|//div[contains(@class, 'tag-list')]/a")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            topic_string = strip_nodes_to_text(self._topic_selector(self.precomputed.doc), join_on=",")
            if topic_string is not None:
                return generic_topic_parsing(topic_string, delimiter=",")
            return []

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                image_selector=XPath("//figure[not(contains(@class, 'teaser'))]//img"),
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath(
                    "./ancestor::figure//*[(self::div and contains(@class, 'caption')) or self::figcaption]"
                ),
                author_selector=re.compile(r"\s*—?\s*©\s*(?P<credits>.*)"),
                lower_boundary_selector=XPath("//div[@class='widget partnerbox_1']"),
            )



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.filter import inverse, regex_filter
from fundus.scraping.url import RSSFeed, Sitemap

from .the_namibian import TheNamibianParser


class NA(metaclass=PublisherGroup):
    default_language = "en"

    TheNamibian = Publisher(
        name="The Namibian",
        domain="https://www.namibian.com.na/",
        parser=TheNamibianParser,
        sources=[
            RSSFeed("https://www.namibian.com.na/feed/", languages={"en", "kj"}),
            Sitemap(
                "https://namibian.com.na/sitemap_index.xml",
                sitemap_filter=inverse(regex_filter("post-sitemap")),
                reverse=True,
                languages={"en", "kj"},
            ),
        ],
        deprecated=True,
    )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)
from fundus.publishers.de.funke import FunkeParser


class WAZParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 2, 21)
        _paragraph_selector: XPath = CSSSelector(".article__body > p")
        _summary_selector: XPath = CSSSelector(".article__header__intro__text")
        _subheadline_selector: XPath = CSSSelector(".article__body > h3")
        _topics_selector = XPath("//div[@class='not-prose  mb-4 mx-5 font-sans']/ul/li")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.meta.get("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def topics(self) -> List[str]:
            authors = generic_author_parsing(self.precomputed.meta.get("author"))
            if topics := generic_topic_parsing(self.precomputed.meta.get("keywords")):
                return [topic for topic in topics if topic not in authors]
            else:
                pass
            return [
                re.sub(r"\s*:.+", "", node.text_content()).strip()
                for node in self._topics_selector(self.precomputed.doc)
            ]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                lower_boundary_selector=XPath("//a[@href='/' and contains(text(), 'Startseite')]"),
                caption_selector=XPath("(./ancestor::figure//figcaption//span)[1]"),
                author_selector=XPath("(./ancestor::figure//figcaption//span)[2]"),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date(2025, 8, 24)

        _paragraph_selector = XPath(
            "//div[@class='article-body'] /p[position()>1 and not(@rel='author' or re:test(string(), '^>>.*[+]{3}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _summary_selector = XPath("//div[@class='article-body'] /p[position()=1]")
        _subheadline_selector = XPath("//div[@class='article-body'] / h3[not(text()='Auch interessant')]")

    class V2(FunkeParser.V1_1):
        # It seems like WAZ was bought by Funke and now uses the same layout
        VALID_UNTIL = datetime.date.today()



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.publishers.be.nieuwsblad import NieuwsbladParser
from fundus.scraping.url import RSSFeed


class BE(metaclass=PublisherGroup):
    default_language = "nl"

    Nieuwsblad = Publisher(
        name="Nieuwsblad",
        domain="https://www.nieuwsblad.be/",
        parser=NieuwsbladParser,
        sources=[
            RSSFeed("https://www.nieuwsblad.be/rss/section/55178e67-15a8-4ddd-a3d8-bfe5708f8932"),
            RSSFeed("https://www.nieuwsblad.be/rss/section/7f1bc231-66e7-49f0-a126-b7346eb3e2fa"),
            RSSFeed("https://www.nieuwsblad.be/rss/section/3dfcee99-2971-4c4c-a603-8c41ae86398b"),
            RSSFeed("https://www.nieuwsblad.be/rss/section/c0c3b215-10be-4f82-86d6-8b8584a5639d"),
        ],
    )



from datetime import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import (
    ArticleBody,
    BaseParser,
    Image,
    ParserProxy,
    attribute,
    utility,
)
from fundus.parser.utility import image_extraction


class EuronewsParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = CSSSelector("p.c-article-summary")
        _subheadline_selector = CSSSelector("div.c-article-content > h2")
        _paragraph_selector = CSSSelector("div.c-article-content > p")

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            article_body = utility.extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )
            return article_body

        @attribute
        def authors(self) -> List[str]:
            author_string = self.precomputed.ld.xpath_search("NewsArticle/author/name")
            return utility.generic_author_parsing(author_string)

        @attribute
        def publishing_date(self) -> Optional[datetime]:
            date_string = self.precomputed.meta.get("date.available")
            return utility.generic_date_parsing(date_string)

        @attribute
        def topics(self) -> List[str]:
            keyword_string = self.precomputed.meta.get("keywords")
            return utility.generic_topic_parsing(keyword_string)

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath(
                    "//img[contains(@class, 'c-article-media__img')" " or contains(@class, 'widgetImage__image')]"
                ),
                caption_selector=XPath(
                    "./ancestor::div[contains(@class, 'c-article-image-video')]"
                    "//div[contains(@class, 'c-article-caption__content')]|"
                    "./ancestor::figure//span[@class='widget__captionText']"
                ),
                author_selector=XPath(
                    "./ancestor::div[contains(@class, 'c-article-image-video')]"
                    "//div[contains(@class, 'c-article-image-copyright')]|"
                    "./ancestor::figure//span[@class='widget__captionCredit']"
                ),
            )



import datetime
import re
from typing import List, Optional, Pattern

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class BhaskarParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//article //p | //article //*[@style='border-bottom:none'] //li")

        _topic_bloat_pattern: Pattern[str] = re.compile(r"news", flags=re.IGNORECASE)

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return [
                topic
                for topic in generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))
                if not re.search(self._topic_bloat_pattern, topic)
            ]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//article//picture//img"),
                upper_boundary_selector=XPath("//article"),
                caption_selector=XPath("(./ancestor::div[@class='f3e032cb']/following-sibling::*[1])/span[text()]"),
            )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class FreiePresseParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 8, 4)
        _summary_selector = CSSSelector("#artikel-content > p.bold")
        _paragraph_selector = XPath(
            "//*[@id='artikel-content']//p[not(ancestor::div[@class='pw-layer'] or @class='bold')]"
        )
        _subheadline_selector = CSSSelector("#artikel-content h2")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            if not (authors := self.precomputed.ld.xpath_search("NewsArticle/author")):
                return []
            else:
                return generic_author_parsing(
                    [author for author in authors if not author == "Chemnitzer Verlag und Druck GmbH & Co. KG"]
                )

        @attribute
        def title(self) -> Optional[str]:
            if title := self.precomputed.meta.get("og:title"):
                return re.sub(r"\s*\|.*", "", title)
            return None

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.ld.bf_search("keywords"), delimiter="/")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath(
                    "((//div[contains(@class,'wrapImg')]//picture)[1])//img | //img[@class='media-image']"
                ),
                lower_boundary_selector=XPath("//div[@class='section-topic']"),
                caption_selector=XPath("./ancestor::li[@class='img gallery-item']//span[@class='img-info']"),
                author_selector=re.compile(r"(?i)bild:(?P<credits>.*)"),
                relative_urls=True,
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()
        _paragraph_selector = CSSSelector("#artikel-content p:not(.bold)")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath(
                    "//div[@class='detail-img__image-wrapper detail-img__image-wrapper--gradient']//img"
                ),
                lower_boundary_selector=CSSSelector("a.article__copyright"),
                caption_selector=XPath(
                    "./ancestor::div[@class='detail-img']//div[@class='detail-img__description no-transition']/div/text()"
                ),
                author_selector=re.compile(r"(?i)bild:(?P<credits>.*)"),
                relative_urls=True,
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class HaberturkParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = XPath("//article//h2[preceding-sibling::h1]")
        _paragraph_selector = CSSSelector("article p")
        _subheadline_Selector = XPath("//article//h2[not(preceding-sibling::h1)]")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_Selector,
            )

        @attribute(validate=False)
        def description(self) -> Optional[str]:
            return self.precomputed.meta.get("og:description")

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(
                self.precomputed.ld.bf_search("datePublished") or self.precomputed.meta.get("datePublished")
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=CSSSelector("article"),
                image_selector=CSSSelector("img[data-zoomable]"),
            )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class AnadoluAjansiParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = CSSSelector("div.detay-bg > div > div > h4")
        _paragraph_selector = XPath(
            "//div[@class='detay-icerik']"
            "//h6[not(ancestor::div[@class='detay-paylas'])] | "
            "//div[@class='detay-icerik']//p"
        )
        _subheadline_Selector = CSSSelector("div.detay-icerik > div:nth-child(2) > h3")
        _author_selector = CSSSelector("div.detay-bg > div > div > div > span:nth-child(1)")
        _date_selector = CSSSelector("div.detay-bg > div > div > div > span.tarih")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_Selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            if authors_list := self._author_selector(self.precomputed.doc):
                if (content := authors_list[0].text) is None:
                    return []
                authors_str = content.replace("|", "")
                return generic_author_parsing(authors_str)
            return []

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            if date_nodes := self._date_selector(self.precomputed.doc):
                if (content := date_nodes[0].text) is None:
                    return None
                match = re.search(r"(\d{2}\.\d{2}\.\d{4})", content)
                if match is None:
                    return None
                return generic_date_parsing(match.group(1))
            return None

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            if keywords_ := (
                generic_topic_parsing(self.precomputed.meta.get("keywords"))
                or generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))
            ):
                if "Anadolu Ajansı" in keywords_:
                    keywords_.remove("Anadolu Ajansı")
                return keywords_
            return []

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=CSSSelector("div.row.detay.container > div.col-md-10 > img," "div img[alt='']"),
                relative_urls=True,
            )



import datetime
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class ABCParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//div[@class='voc-d ']//p[@class='voc-p']")
        _subheadline_selector = XPath("//div[@class='voc-d ']//h3[@class='voc-d-c__s-title']")
        _summary_selector = XPath("//div[@class='voc-info-container']/h2[text()]")

        _topics_selector = XPath("//div[@class='voc-wrapper']//ul[@class='voc-topics__list']/li[position() > 1]")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("title")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def topics(self) -> List[str]:
            return [node.text_content().strip() for node in self._topics_selector(self.precomputed.doc)]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure//img[@class='voc-img']"),
                upper_boundary_selector=XPath("//article"),
                caption_selector=XPath(
                    "./ancestor::div[contains(@class, 'voc-img-container')]//figcaption/span[contains(@class,'text')]"
                ),
                author_selector=XPath(
                    "./ancestor::div[contains(@class, 'voc-img-container')]//figcaption/span[contains(@class,'author')]"
                ),
            )



from __future__ import annotations

from typing import Protocol, runtime_checkable


@runtime_checkable
class Delay(Protocol):
    """Protocol to define crawl delays between batches."""

    def __call__(self) -> float:
        """Yields a float specifying the minimum crawler delay for the current article batch in seconds.

        The effective delay does include crawling execution time between batches,
        i.e. the effective delay is max(execution_time, delay).

        Examples:
            >>> import random
            >>> delay: Delay = lambda: random.random()
            Will use a random delay in [0, 1) seconds.

        Returns:
            float: The delay time in seconds.

        """
        ...



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.url import RSSFeed, Sitemap

from .lrt import LRTParser

# noinspection PyPep8Naming


class LT(metaclass=PublisherGroup):
    default_language = "lt"

    LRT = Publisher(
        name="Lithuanian National Radio and Television",
        domain="https://www.lrt.lt",
        parser=LRTParser,
        sources=[
            Sitemap("https://www.lrt.lt/servisai/sitemap/sitemap-index.xml"),
            RSSFeed("https://www.lrt.lt/?rss"),
        ],
    )



import logging
from typing import Dict, Set, cast

from fundus.utils.serialization import JSONVal

_default_handler_level = logging.ERROR

__all__ = ["set_log_level", "add_handler", "create_logger", "loggers", "handlers"]

# create std-handler
_stream_handler = logging.StreamHandler()
_stream_handler.name = "std-handler"
_formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
_stream_handler.setFormatter(_formatter)
_stream_handler.setLevel(_default_handler_level)

loggers: Dict[str, logging.Logger] = {}
handlers: Dict[str, logging.Handler] = {_stream_handler.name: _stream_handler}


def create_logger(name: str) -> logging.Logger:
    """Create a new logger with as <name>.

    Per defaults the loggers' log level is set to DEBUG and the following handlers will be added
    automatically (see <handlers> for more details):

    std-handler: StreamHandler | ERROR | %(asctime)s - %(name)s - %(levelname)s - %(message)s

    Args:
        name: Reference name for the created logger.

    Returns:
        A new logger with name <name>
    """
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    for handler in handlers.values():
        logger.addHandler(handler)
    loggers[name] = logger
    return logger


def set_log_level(level: int):
    """Set log level for all handlers.

    Args:
        level: The new log level to set
    """
    for handler in handlers.values():
        handler.setLevel(level)


def add_handler(handler: logging.Handler):
    """Add a new handler to all logger.

    Args:
        handler: The new handler to add.
    """
    if handler.name is None:
        raise ValueError(f"Handlers to add must have a name set")

    if handlers.get(handler.name) is not None:
        raise ValueError(f"Handler with name {handler.name} already exists")

    handlers[handler.name] = handler
    for logger in loggers.values():
        logger.addHandler(handler)


def get_current_config() -> JSONVal:
    """Get the current logging configuration as JSON.

    Returns:
        The current logging configuration as JSON.
    """

    formatters: Set[logging.Formatter] = cast(
        Set[logging.Formatter], {handler.formatter for handler in handlers.values()}
    )

    def get_formatter_config(formatter: logging.Formatter) -> JSONVal:
        return {"format": formatter._fmt}

    def get_handler_config(handler: logging.Handler) -> JSONVal:
        config: Dict[str, JSONVal] = {
            "level": handler.level,
            "formatter": hex(id(handler.formatter)),
            "class": handler.__class__.__module__ + "." + handler.__class__.__name__,
        }
        if isinstance(handler, logging.FileHandler):
            config["filename"] = handler.baseFilename
            config["mode"] = handler.mode
            if handler.encoding is not None:
                config["encoding"] = handler.encoding
            config["delay"] = handler.delay
        return config

    def get_logger_config(logger: logging.Logger) -> JSONVal:
        return {
            "level": logger.level,
            "handlers": [handler.name for handler in logger.handlers],
            "propagate": logger.propagate,
        }

    return {
        "version": 1,
        "formatters": {hex(id(formatter)): get_formatter_config(formatter) for formatter in formatters},
        "handlers": {str(handler.name): get_handler_config(handler) for handler in handlers.values()},
        "loggers": {logger.name: get_logger_config(logger) for logger in loggers.values()},
    }



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_nodes_to_text,
    generic_topic_parsing,
    image_extraction,
)


class LuxemburgerWortParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//p[contains(@class, 'articleParagraph')]")
        _summary_selector = XPath("//h2[contains(@class, 'articleParagraph')]")
        _subheadline_selector = XPath("//h4[contains(@class, 'articleSubheading')]")

        _topic_selector = XPath("//div[contains(@class, 'tag-list')]//a")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(generic_nodes_to_text(self._topic_selector(self.precomputed.doc)))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure[not(contains(@class, 'Teaser'))]//img"),
                upper_boundary_selector=CSSSelector("h1"),
                caption_selector=XPath("./ancestor::figure//div[contains(@class, 'ImageCaption')]"),
                author_selector=re.compile(r"(?i)Foto:\s*(?P<credits>.*)"),
            )



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class LaVanguardiaParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//div[@class='article-modules']//p[@class='paragraph'] | "
            "//div[@class='widget' and not(@id)]//p[not(@class='creditos')]"
        )
        _subheadline_selector = XPath(
            "//div[@class='article-modules']//h3[@class='subtitle'] | "
            "//div[@class='widget' and not(@id)]//h2|//span[@class='ubicacion']"
        )
        _summary_selector = XPath("//h2[@class='epigraph']|//div[@id='slide-content-1']/p")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("title")

        @attribute
        def authors(self) -> List[str]:
            return [
                re.sub(r"(?u)\s*\u200b.*", "", author)
                for author in generic_author_parsing(self.precomputed.ld.bf_search("author"))
            ]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure[contains(@class,'composite-image')]//img"),
                caption_selector=XPath("./ancestor::figure//figcaption/p"),
                author_selector=XPath("./ancestor::figure//figcaption/span"),
                relative_urls=True,
            )



import datetime
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class NineNewsParser(ParserProxy):
    class V1(BaseParser):
        # _paragraph_selector = CSSSelector("div.article__body div.block-content")

        _bloat_regex = r"^READ MORE:"
        _paragraph_selector = XPath(
            f"//div[@class='article__body'] "
            f"//div[@class='block-content'] "
            f"/div[child::span and not(re:test(string(), '{_bloat_regex}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _subheadline_selector = XPath("//div[@class='article__body'] //div[@class='block-content'] /div[child::h3]")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//figcaption/text()[1]"),
                author_selector=XPath("./ancestor::figure//figcaption/text()[last()]"),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.data import Image
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class BRParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 8, 26)

        _summary_selector = XPath(
            "//div[starts-with(@class, 'ArticleHeader_section')]//p[starts-with(@class, 'ArticleModuleTeaser_teaserText') or starts-with(@class, 'ArticleItemTeaserText_text')]"
        )
        # paragraph_selector captures div contents for full article, p for teaser-only flash articles
        _paragraph_selector = XPath("(//div[starts-with(@class, 'ArticleModuleText_content')])[1]//p")

        _subheadline_selector = XPath(
            "//section[starts-with(@class, 'ArticleModuleText_wrapper')]//div[starts-with(@class, 'ArticleModuleText_content')]//h2"
        )

        @attribute
        def title(self) -> Optional[str]:
            return title if isinstance(title := self.precomputed.ld.bf_search("headline"), str) else None

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))

        @attribute
        def images(self) -> List[Image]:
            author_pattern: str = r"(?<=\|\sBild:\s).*$"
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure[not(parent::aside)]//img"),
                author_selector=XPath(
                    f"re:match(./@title, '{author_pattern}')",
                    namespaces={"re": "http://exslt.org/regular-expressions"},
                ),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()

        _bloat_pattern = "Das ist die Europäische Perspektive bei BR24."
        _summary_selector = XPath("//header //p[@class='body3 ArticleItemTeaserText_text__H_RS_']")
        _subheadline_selector = XPath("//section[@id='articlebody'] //h2[text()]")
        _paragraph_selector = XPath(
            f"//section[@id='articlebody'] //section[@class='ArticleModuleText_wrapper__AyX6M'] //p[text() and not(re:test(string(), '{_bloat_pattern}'))] |"
            "//section[@id='articlebody'] //section[@class='ArticleModuleText_wrapper__AyX6M'] //li |"
            "//section[@class='ShortnewsDetail_content__79bZq'] //p[1]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )

        _date_selector = CSSSelector("p.ShortnewsDetail_source__2ep85.heading4")

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            if date_nodes := self._date_selector(self.precomputed.doc):
                if (content := date_nodes[0].text) is None:
                    return None
                date_string = content.split(",")[-1]
                tz_aware_date = date_string.replace("Uhr", "+02:00")
                return generic_date_parsing(tz_aware_date)
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.url import Sitemap

from ...scraping.filter import inverse, regex_filter
from .wochenblatt import WochenblattParser


class PY(metaclass=PublisherGroup):
    default_language = "es"

    Wochenblatt = Publisher(
        name="Wochenblatt",
        domain="https://wochenblatt.cc/",
        parser=WochenblattParser,
        sources=[
            Sitemap(
                "https://wochenblatt.cc/sitemap_index.xml",
                sitemap_filter=inverse(regex_filter("post-")),
                reverse=True,
                languages={"de"},
            ),
        ],
    )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath
from lxml.html import fromstring, tostring

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    apply_substitution_pattern_over_list,
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class TimesOfIndiaParser(ParserProxy):
    class V1(BaseParser):
        _subheadline_selector = XPath(
            "(//div[@class='_s30J clearfix  '])[1]/div/b |"
            "(//div[@class='_s30J clearfix  '])[1]/div/h2 |"
            "(//div[@class='_s30J clearfix  '])[1]//span[contains(class, strong)]"
        )
        _paragraph_selector = XPath("(//div[@class='_s30J clearfix  '])[1]/p[text()]")
        _summary_selector = XPath("//div[@class='M1rHh undefined']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            html_as_string = tostring(self.precomputed.doc).decode("utf-8")
            html_as_string = re.sub(r"(</div>)((\r\n|\r|\n)<br>)", "</div><p>", html_as_string)
            html_as_string = re.sub(r"</div>\s*</div>(?!<)", "</div></div><p>", html_as_string)
            html_as_string = re.sub(r"</div>\s*</div>\s*</div>(?!<)", "</div></div></div><p>", html_as_string)
            html_as_string = re.sub(r"<br>(\r\n|\r|\n)(:?<div)", "</p>", html_as_string)
            html_as_string = re.sub(r"(:?::before)(\r\n|\r|\n)", "<p>", html_as_string)
            html_as_string = re.sub(r"(\r\n|\r|\n)(:?::after)", "</p>", html_as_string)
            html_as_string = re.sub(r"<br>", "</p><p>", html_as_string)
            html_as_string = re.sub(
                r"<div class=\"_s30J clearfix  \">", "<div class=\"_s30J clearfix  \"><p class='intro'>", html_as_string
            )
            return extract_article_body_with_selector(
                fromstring(html_as_string),  # type: ignore
                summary_selector=self._summary_selector,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return apply_substitution_pattern_over_list(
                generic_author_parsing(self.precomputed.ld.bf_search("author")),
                re.compile(r"(TOI .*|TIMESOFINDIA.COM)"),
            )

        @attribute
        def title(self) -> Optional[str]:
            if title := self.precomputed.meta.get("og:title"):
                return re.sub(r"( - Times.*| \| (India.*|.*News))", "", title)
            return None

        @attribute
        def topics(self) -> List[str]:
            return [
                topic.title()
                for topic in generic_topic_parsing(self.precomputed.meta.get("news_keywords"))
                if "News" not in topic.title()
            ]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=CSSSelector("div.contentwrapper.clearfix"),
                lower_boundary_selector=CSSSelector("div.authorComment"),
                image_selector=CSSSelector("section.leadmedia img"),
                caption_selector=XPath(
                    "./ancestor::section[contains(@class, 'leadmedia')]//div[contains(@class, 'img_cptn')]"
                ),
                author_selector=re.compile(r"\((?P<credits>.*?)\)$"),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_nodes_to_text,
    generic_topic_parsing,
    image_extraction,
)


class SternParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 10, 26)

        _paragraph_selector = CSSSelector(".article__body >p")
        _summary_selector = CSSSelector(".intro__text")
        _subheadline_selector = CSSSelector(".subheadline-element")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            initial_authors = generic_author_parsing(self.precomputed.ld.bf_search("author"))
            return [el for el in initial_authors if el != "STERN.de"]

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(
                self.precomputed.meta.get(
                    "date",
                )
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            topic_nodes = self.precomputed.doc.cssselect(".article__tags li.links__item")
            return [node.text_content().strip("\n ") for node in topic_nodes]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                image_selector=XPath("//figure[not(contains(@class, 'teaser'))]//img"),
                paragraph_selector=self._paragraph_selector,
                lower_boundary_selector=CSSSelector(".article__tags li.links__item"),
                caption_selector=XPath("./ancestor::figure//figcaption//div[contains(@class,'caption')]"),
                author_selector=XPath("./ancestor::figure//figcaption//div[contains(@class,'credits')]"),
            )

    class V2(BaseParser):
        _paragraph_selector = CSSSelector(".article__body > .text-element > p.is-initial")
        _summary_selector = CSSSelector(".article__body > .intro")
        _subheadline_selector = CSSSelector(".article__body > .subheadline-element")

        _topic_selector = CSSSelector("ul.tags > li")
        _author_selector = CSSSelector("li.authors__list-item > a, li.authors__list-item > .typo-article-info-bold")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(generic_nodes_to_text(self._author_selector(self.precomputed.doc)))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(
                generic_nodes_to_text(self._topic_selector(self.precomputed.doc), normalize=True)
            )

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                image_selector=XPath("//figure[not(contains(@class, 'teaser'))]//img"),
                paragraph_selector=self._paragraph_selector,
                lower_boundary_selector=self._topic_selector,
                caption_selector=XPath("./ancestor::figure//figcaption//div[contains(@class,'caption')]"),
                author_selector=XPath("./ancestor::figure//figcaption//div[contains(@class,'credits')]"),
            )



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.filter import inverse, regex_filter
from fundus.scraping.url import Sitemap

from .israel_nachrichten import IsraelNachrichtenParser


class IL(metaclass=PublisherGroup):
    default_language = "he"

    IsraelNachrichten = Publisher(
        name="Israel Nachrichten",
        domain="https://www.israel-nachrichten.org/",
        parser=IsraelNachrichtenParser,
        sources=[
            Sitemap(
                "https://www.israel-nachrichten.org/wp-sitemap.xml",
                reverse=True,
                sitemap_filter=inverse(regex_filter("posts-post")),
                languages={"de"},
            )
        ],
    )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class TheIndependentParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = CSSSelector("article div[id='main'] > p")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            body = extract_article_body_with_selector(self.precomputed.doc, paragraph_selector=self._paragraph_selector)
            return body

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=CSSSelector("figure > div > img, div[data-gallery-length] > img"),
                upper_boundary_selector=CSSSelector("article"),
                author_selector=re.compile(r"(?P<credits>(\([^)]*\)\s?)+$)"),
            )



from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.url import NewsMap, RSSFeed, Sitemap

from ...scraping.filter import inverse, regex_filter
from .nzz import NZZParser
from .srf import SRFParser
from .ta import TAParser
from .zwanzig_minuten import ZwanzigMinutenParser

# noinspection PyPep8Naming


class CH(metaclass=PublisherGroup):
    default_language = "de"

    SRF = Publisher(
        name="Schweizer Radio und Fernsehen",
        domain="https://www.srf.ch/",
        parser=SRFParser,
        sources=[
            RSSFeed("https://www.srf.ch/news/bnf/rss/1646"),
            NewsMap("https://www.srf.ch/sitemaps/newsmap/news/index.xml"),
            Sitemap("https://www.srf.ch/new-news-sitemap"),
        ],
    )

    NZZ = Publisher(
        name="Neue Zürcher Zeitung (NZZ)",
        domain="https://www.nzz.ch/",
        parser=NZZParser,
        sources=[
            NewsMap("https://www.nzz.ch/sitemap/news0.xml"),
            Sitemap(
                "https://www.nzz.ch/sitemap.xml",
                sitemap_filter=inverse(regex_filter(r"sitemap/[\d]{4}/[\d]{2}")),
            ),
        ],
    )

    TagesAnzeiger = Publisher(
        name="Tages-Anzeiger",
        domain="https://www.tagesanzeiger.ch/",
        parser=TAParser,
        sources=[
            NewsMap("https://www.tagesanzeiger.ch/sitemaps/news.xml"),
            Sitemap(
                "https://www.tagesanzeiger.ch/sitemaps/sitemapindex.xml",
                reverse=True,
                sitemap_filter=regex_filter("news|category"),
            ),
        ],
    )
    ZwanzigMinuten = Publisher(
        name="Zwanzig Minuten",
        domain="https://www.20min.ch/",
        parser=ZwanzigMinutenParser,
        sources=[
            NewsMap("https://www.20min.ch/sitemaps/de/news.xml"),
            Sitemap("https://www.20min.ch/sitemaps/de/articles.xml"),
            NewsMap("https://www.20min.ch/sitemaps/fr/news.xml"),
            Sitemap("https://www.20min.ch/sitemaps/fr/articles.xml"),
        ],
    )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute, function
from fundus.parser.data import ArticleSection, Image, TextSequence
from fundus.parser.utility import (
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
    parse_json,
)


class WestAustralianParser(ParserProxy):
    class V1(BaseParser):
        _page_data_selector = XPath(
            "string(//script[re:test(text(), 'window.PAGE_DATA')])",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )

        @function(priority=1)
        def _parse_page_content(self):
            if not (parsed_json := parse_json(self._page_data_selector(self.precomputed.doc))):
                raise ValueError("Couldn't parse page data")
            self.precomputed.ld.add_ld(parsed_json, "windows.PAGE_DATA")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            content_blocks = self.precomputed.ld.xpath_search(XPath("//publication/content/blocks"))
            paragraphs = []
            for block in content_blocks:
                if block.get("kind") == "text" and (text := block.get("text")):
                    paragraphs.append(text)
            section = ArticleSection(headline=TextSequence([]), paragraphs=TextSequence(paragraphs))
            return ArticleBody(summary=TextSequence([]), sections=[section])

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=CSSSelector("div#ArticleContent > p"),
                upper_boundary_selector=CSSSelector("article"),
                lower_boundary_selector=CSSSelector("div#footer"),
                caption_selector=XPath("./ancestor::figure //span[contains(@class, 'CaptionText')] /span[1]"),
                author_selector=XPath("./ancestor::figure //span[contains(@class, 'CaptionText')] /span[last()]"),
            )



import datetime
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
    strip_nodes_to_text,
)


class RzeczpospolitaParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//div[contains(@class,'article--content')]//div[contains(@class,'body articleBody')]"
            "//p[contains(@class, 'article--paragraph')]"
        )
        _summary_selector = XPath("//div[@class='blog--subtitle ']")
        _subheadline_selector = XPath(
            "//div[contains(@class,'article--content')]//div[contains(@class,'body articleBody')]//h2"
        )

        _topic_selector = XPath("//div[@data-mrf-section='Article / Tags']/a")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            topic_string = strip_nodes_to_text(self._topic_selector(self.precomputed.doc), join_on=",")
            if topic_string is not None:
                return generic_topic_parsing(topic_string, delimiter=",")
            return []

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//div[@class='blog--image']//img"),
                upper_boundary_selector=XPath("//div[@class='row']//h1"),
                caption_selector=XPath("./ancestor::div[@class='blog--image']//p[@class='article--media--lead']"),
                author_selector=XPath("./ancestor::div[@class='blog--image']//p[@class='image--author']"),
            )



import datetime
import re
from typing import List, Optional, Pattern

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.data import Image
from fundus.parser.utility import (
    apply_substitution_pattern_over_list,
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class NZZParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = CSSSelector("div.headline p.headline__lead")
        _subheadline_selector = CSSSelector("div.article h2.subtitle, div.article h5.articlecomponent")
        _paragraph_selector = CSSSelector(
            "div.article section[data-nzz-tid='article'] p.articlecomponent:not(.footnote), "
            "div.article div.articlecomponent:not(.content-table) li"
        )

        _author_substitution_pattern: Pattern[str] = re.compile(r"\(.+\)$")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return apply_substitution_pattern_over_list(
                generic_author_parsing(self.precomputed.ld.bf_search("author")),
                pattern=self._author_substitution_pattern,
                replacement="",
            )

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.meta.get("date"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("title")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//h2"),
                author_selector=XPath("./ancestor::figure//div[@class='image-description__author']"),
                upper_boundary_selector=CSSSelector("div#page"),
                lower_boundary_selector=XPath("//div[@class='sharebox']"),
            )



import datetime
import re
from typing import List, Optional, Union

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class MetroParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 11, 17)
        _summary_selector = XPath("//article / div[@class='article-body'] / p[1]")
        _subheadline_selector: Union[CSSSelector, XPath] = CSSSelector("article > div.article-body > h2")

        _bloat_regex_ = (
            r"^Got a story|"
            r"^Get in touch with our news team|"
            r"^Get in touch by emailing|"
            r"^If you’ve got a celebrity story|"
            r"^For more stories|"
            r"^Follow Metro|"
            r"^\s*MORE :|"
            r"^Share your views in the comments|"
            r"^Email gamecentral@metro.co.uk|"
            r"^To submit Inbox letters and Reader’s Features more easily|"
            r"^Do you have a story to share?"
        )
        _paragraph_selector = XPath(
            f"//article "
            f"/div[@class='article-body'] "
            f"/p[position()>1 and not(re:test(string(), '{_bloat_regex_}'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )

        @attribute
        def body(self) -> Optional[ArticleBody]:
            body = extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )
            return body

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("article:tag"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=XPath("//article"),
                author_selector=re.compile(r"(?P<credits>\([^(]+\)$)"),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()
        _summary_selector = XPath("//article//div[@class='article__content__inner']/p[1]")
        _paragraph_selector = XPath("//article//div[@class='article__content__inner']/p[not(@class) and position()>1]")
        _subheadline_selector = XPath("//article//div[@class='article__content__inner']/h2")
